@@ -3,7 +3,7 @@
  * \brief abstraction for all consumer services
  * \see mlt_consumer_s
  *
- * Copyright (C) 2003-2020 Meltytech, LLC
+ * Copyright (C) 2003-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -80,16 +80,10 @@ typedef struct
 }
 consumer_private;
 
-typedef void* ( *thread_function_t )( void* );
-
-static void mlt_consumer_frame_render( mlt_listener listener, mlt_properties owner, mlt_service self, void **args );
-static void mlt_consumer_frame_show( mlt_listener listener, mlt_properties owner, mlt_service self, void **args );
-static void mlt_consumer_property_changed( mlt_properties owner, mlt_consumer self, char *name );
+static void mlt_consumer_property_changed(mlt_properties owner, mlt_consumer self, mlt_event_data );
 static void apply_profile_properties( mlt_consumer self, mlt_profile profile, mlt_properties properties );
-static void on_consumer_frame_show( mlt_properties owner, mlt_consumer self, mlt_frame frame );
-static void transmit_thread_create( mlt_listener listener, mlt_properties owner, mlt_service self, void **args );
-static void mlt_thread_create( mlt_consumer self, thread_function_t function );
-static void transmit_thread_join( mlt_listener listener, mlt_properties owner, mlt_service self, void **args );
+static void on_consumer_frame_show(mlt_properties owner, mlt_consumer self, mlt_event_data );
+static void mlt_thread_create( mlt_consumer self, mlt_thread_function_t function );
 static void mlt_thread_join( mlt_consumer self );
 static void consumer_read_ahead_start( mlt_consumer self );
 
@@ -149,14 +143,14 @@ int mlt_consumer_init( mlt_consumer self, void *child, mlt_profile profile )
 		priv->image_format = mlt_image_yuv422;
 		priv->audio_format = mlt_audio_s16;
 
-		mlt_events_register( properties, "consumer-frame-show", ( mlt_transmitter )mlt_consumer_frame_show );
-		mlt_events_register( properties, "consumer-frame-render", ( mlt_transmitter )mlt_consumer_frame_render );
-		mlt_events_register( properties, "consumer-thread-started", NULL );
-		mlt_events_register( properties, "consumer-thread-stopped", NULL );
-		mlt_events_register( properties, "consumer-stopping", NULL );
-		mlt_events_register( properties, "consumer-stopped", NULL );
-		mlt_events_register( properties, "consumer-thread-create", ( mlt_transmitter )transmit_thread_create );
-		mlt_events_register( properties, "consumer-thread-join", ( mlt_transmitter )transmit_thread_join );
+		mlt_events_register( properties, "consumer-frame-show" );
+		mlt_events_register( properties, "consumer-frame-render" );
+		mlt_events_register( properties, "consumer-thread-started" );
+		mlt_events_register( properties, "consumer-thread-stopped" );
+		mlt_events_register( properties, "consumer-stopping" );
+		mlt_events_register( properties, "consumer-stopped" );
+		mlt_events_register( properties, "consumer-thread-create" );
+		mlt_events_register( properties, "consumer-thread-join" );
 		mlt_events_listen( properties, self, "consumer-frame-show", ( mlt_listener )on_consumer_frame_show );
 
 		// Register a property-changed listener to handle the profile property -
@@ -208,9 +202,10 @@ static void apply_profile_properties( mlt_consumer self, mlt_profile profile, ml
  * \param name the name of the property that changed
  */
 
-static void mlt_consumer_property_changed( mlt_properties owner, mlt_consumer self, char *name )
+static void mlt_consumer_property_changed( mlt_properties owner, mlt_consumer self, mlt_event_data event_data )
 {
-	if ( !strcmp( name, "mlt_profile" ) )
+	const char *name = mlt_event_data_to_string(event_data);
+	if ( name && !strcmp( name, "mlt_profile" ) )
 	{
 		// Get the properties
 		mlt_properties properties = MLT_CONSUMER_PROPERTIES( self );
@@ -330,40 +325,6 @@ static void mlt_consumer_property_changed( mlt_properties owner, mlt_consumer se
 	}
 }
 
-/** The transmitter for the consumer-frame-show event
- *
- * Invokes the listener.
- *
- * \private \memberof mlt_consumer_s
- * \param listener a function pointer that will be invoked
- * \param owner the events object that will be passed to \p listener
- * \param self a service that will be passed to \p listener
- * \param args an array of pointers - the first entry is passed as a frame to \p listener
- */
-
-static void mlt_consumer_frame_show( mlt_listener listener, mlt_properties owner, mlt_service self, void **args )
-{
-	if ( listener != NULL )
-		listener( owner, self, ( mlt_frame )args[ 0 ] );
-}
-
-/** The transmitter for the consumer-frame-render event
- *
- * Invokes the listener.
- *
- * \private \memberof mlt_consumer_s
- * \param listener a function pointer that will be invoked
- * \param owner the events object that will be passed to \p listener
- * \param self a service that will be passed to \p listener
- * \param args an array of pointers - the first entry is passed as a frame to \p listener
- */
-
-static void mlt_consumer_frame_render( mlt_listener listener, mlt_properties owner, mlt_service self, void **args )
-{
-	if ( listener != NULL )
-		listener( owner, self, ( mlt_frame )args[ 0 ] );
-}
-
 /** A listener on the consumer-frame-show event
  *
  * Saves the position of the frame shown.
@@ -374,8 +335,9 @@ static void mlt_consumer_frame_render( mlt_listener listener, mlt_properties own
  * \param frame the frame that was shown
  */
 
-static void on_consumer_frame_show( mlt_properties owner, mlt_consumer consumer, mlt_frame frame )
+static void on_consumer_frame_show( mlt_properties owner, mlt_consumer consumer, mlt_event_data event_data )
 {
+	mlt_frame frame = mlt_event_data_to_frame(event_data);
 	if ( frame ) {
 		consumer_private* priv = consumer->local;
 		pthread_mutex_lock( &priv->position_mutex );
@@ -497,11 +459,11 @@ static void set_image_format( mlt_consumer self )
 		priv->image_format = mlt_image_format_id( format );
 		if ( mlt_image_invalid == priv->image_format )
 			priv->image_format = mlt_image_yuv422;
-		// mlt_image_glsl is for internal use only.
+		// mlt_image_movit is for internal use only.
 		// Remapping it glsl_texture prevents breaking existing apps
 		// using the legacy "glsl" name.
-		else if ( mlt_image_glsl == priv->image_format )
-			priv->image_format = mlt_image_glsl_texture;
+		else if ( mlt_image_movit == priv->image_format )
+			priv->image_format = mlt_image_opengl_texture;
 	}
 }
 
@@ -810,7 +772,7 @@ static void *consumer_read_ahead_thread( void *arg )
 	set_audio_format( self );
 	set_image_format( self );
 
-	mlt_events_fire( properties, "consumer-thread-started", NULL );
+	mlt_events_fire( properties, "consumer-thread-started", mlt_event_data_none() );
 
 	// Get the first frame
 	frame = mlt_consumer_get_frame( self );
@@ -828,7 +790,7 @@ static void *consumer_read_ahead_thread( void *arg )
 		// Get the image of the first frame
 		if ( !video_off )
 		{
-			mlt_events_fire( MLT_CONSUMER_PROPERTIES( self ), "consumer-frame-render", frame, NULL );
+			mlt_events_fire( MLT_CONSUMER_PROPERTIES( self ), "consumer-frame-render", mlt_event_data_from_frame(frame) );
 			mlt_frame_get_image( frame, &image, &priv->image_format, &width, &height, 0 );
 		}
 
@@ -906,7 +868,7 @@ static void *consumer_read_ahead_thread( void *arg )
 				height = mlt_properties_get_int( properties, "height" );
 
 				// Get the image
-				mlt_events_fire( MLT_CONSUMER_PROPERTIES( self ), "consumer-frame-render", frame, NULL );
+				mlt_events_fire( MLT_CONSUMER_PROPERTIES( self ), "consumer-frame-render", mlt_event_data_from_frame(frame) );
 				mlt_log_timings_begin();
 				mlt_frame_get_image( frame, &image, &priv->image_format, &width, &height, 0 );
 				mlt_log_timings_end( NULL, "mlt_frame_get_image" );
@@ -991,7 +953,7 @@ static void *consumer_read_ahead_thread( void *arg )
 	priv->queue = NULL;
 	pthread_mutex_unlock( &priv->queue_mutex );
 
-	mlt_events_fire( MLT_CONSUMER_PROPERTIES(self), "consumer-thread-stopped", NULL );
+	mlt_events_fire( MLT_CONSUMER_PROPERTIES(self), "consumer-thread-stopped", mlt_event_data_none() );
 
 	return NULL;
 }
@@ -1053,7 +1015,7 @@ static void *consumer_worker_thread( void *arg )
 	if ( preview_off && preview_format != 0 )
 		format = preview_format;
 
-	mlt_events_fire( properties, "consumer-thread-started", NULL );
+	mlt_events_fire( properties, "consumer-thread-started", mlt_event_data_none() );
 
 	// Continue to read ahead
 	while ( priv->ahead )
@@ -1099,7 +1061,7 @@ static void *consumer_worker_thread( void *arg )
 			// Fetch width/height again
 			width = mlt_properties_get_int( properties, "width" );
 			height = mlt_properties_get_int( properties, "height" );
-			mlt_events_fire( MLT_CONSUMER_PROPERTIES( self ), "consumer-frame-render", frame, NULL );
+			mlt_events_fire( MLT_CONSUMER_PROPERTIES( self ), "consumer-frame-render", mlt_event_data_from_frame(frame) );
 			mlt_frame_get_image( frame, &image, &format, &width, &height, 0 );
 		}
 		mlt_properties_set_int( MLT_FRAME_PROPERTIES( frame ), "rendered", 1 );
@@ -1140,7 +1102,7 @@ static void consumer_read_ahead_start( mlt_consumer self )
 	pthread_cond_init( &priv->queue_cond, NULL );
 
 	// Create the read ahead
-	mlt_thread_create( self, (thread_function_t) consumer_read_ahead_thread );
+	mlt_thread_create( self, (mlt_thread_function_t) consumer_read_ahead_thread );
 	priv->started = 1;
 }
 
@@ -1241,7 +1203,7 @@ static void consumer_read_ahead_stop( mlt_consumer self )
 	{
 		// Inform thread to stop
 		priv->ahead = 0;
-		mlt_events_fire( MLT_CONSUMER_PROPERTIES(self), "consumer-stopping", NULL );
+		mlt_events_fire( MLT_CONSUMER_PROPERTIES(self), "consumer-stopping", mlt_event_data_none() );
 
 		// Broadcast to the condition in case it's waiting
 		pthread_mutex_lock( &priv->queue_mutex );
@@ -1280,7 +1242,7 @@ static void consumer_work_stop( mlt_consumer self )
 	{
 		// Inform thread to stop
 		priv->ahead = 0;
-		mlt_events_fire( MLT_CONSUMER_PROPERTIES(self), "consumer-stopping", NULL );
+		mlt_events_fire( MLT_CONSUMER_PROPERTIES(self), "consumer-stopping", mlt_event_data_none() );
 
 		// Broadcast to the queue condition in case it's waiting
 		pthread_mutex_lock( &priv->queue_mutex );
@@ -1321,7 +1283,7 @@ static void consumer_work_stop( mlt_consumer self )
 		mlt_deque_close( priv->queue );
 		mlt_deque_close( priv->worker_threads );
 
-		mlt_events_fire( MLT_CONSUMER_PROPERTIES(self), "consumer-thread-stopped", NULL );
+		mlt_events_fire( MLT_CONSUMER_PROPERTIES(self), "consumer-thread-stopped", mlt_event_data_none() );
 	}
 }
 
@@ -1603,7 +1565,7 @@ mlt_frame mlt_consumer_rt_frame( mlt_consumer self )
 		if ( !priv->ahead )
 		{
 			priv->ahead = 1;
-			mlt_events_fire( properties, "consumer-thread-started", NULL );
+			mlt_events_fire( properties, "consumer-thread-started", mlt_event_data_none() );
 		}
 		// Get the frame in non real time
 		frame = mlt_consumer_get_frame( self );
@@ -1630,7 +1592,7 @@ mlt_frame mlt_consumer_rt_frame( mlt_consumer self )
 void mlt_consumer_stopped( mlt_consumer self )
 {
 	mlt_properties_set_int( MLT_CONSUMER_PROPERTIES( self ), "running", 0 );
-	mlt_events_fire( MLT_CONSUMER_PROPERTIES( self ), "consumer-stopped", NULL );
+	mlt_events_fire( MLT_CONSUMER_PROPERTIES( self ), "consumer-stopped", mlt_event_data_none() );
 	mlt_event_unblock( ( ( consumer_private* ) self->local )->event_listener );
 }
 
@@ -1766,14 +1728,7 @@ mlt_position mlt_consumer_position( mlt_consumer consumer )
 	return result;
 }
 
-static void transmit_thread_create( mlt_listener listener, mlt_properties owner, mlt_service self, void **args )
-{
-	if ( listener )
-		listener( owner, self,
-			(void**) args[0] /* handle */, (int*) args[1] /* priority */, (thread_function_t) args[2], (void*) args[3] /* data */ );
-}
-
-static void mlt_thread_create( mlt_consumer self, thread_function_t function )
+static void mlt_thread_create(mlt_consumer self, mlt_thread_function_t function )
 {
 	consumer_private *priv = self->local;
 	mlt_properties properties = MLT_CONSUMER_PROPERTIES( self );
@@ -1782,8 +1737,13 @@ static void mlt_thread_create( mlt_consumer self, thread_function_t function )
 	{
 		struct sched_param priority;
 		priority.sched_priority = mlt_properties_get_int( MLT_CONSUMER_PROPERTIES( self ), "priority" );
-		if ( mlt_events_fire( properties, "consumer-thread-create",
-		     &priv->ahead_thread, &priority.sched_priority, function, self, NULL ) < 1 )
+		mlt_event_data_thread data = {
+		    .thread = &priv->ahead_thread,
+		    .priority = &priority.sched_priority,
+		    .function = function,
+		    .data = self
+		};
+		if ( mlt_events_fire( properties, "consumer-thread-create", mlt_event_data_from_object(&data) ) < 1 )
 		{
 			pthread_attr_t thread_attributes;
 			pthread_attr_init( &thread_attributes );
@@ -1801,8 +1761,13 @@ static void mlt_thread_create( mlt_consumer self, thread_function_t function )
 	else
 	{
 		int priority = -1;
-		if ( mlt_events_fire( properties, "consumer-thread-create",
-		     &priv->ahead_thread, &priority, function, self, NULL ) < 1 )
+		mlt_event_data_thread data = {
+		    .thread = &priv->ahead_thread,
+		    .priority = &priority,
+		    .function = function,
+		    .data = self
+		};
+		if ( mlt_events_fire( properties, "consumer-thread-create", mlt_event_data_from_object(&data) ) < 1 )
 		{
 			priv->ahead_thread = malloc( sizeof( pthread_t ) );
 			pthread_t *handle = priv->ahead_thread;
@@ -1811,16 +1776,11 @@ static void mlt_thread_create( mlt_consumer self, thread_function_t function )
 	}
 }
 
-static void transmit_thread_join( mlt_listener listener, mlt_properties owner, mlt_service self, void **args )
-{
-	if ( listener )
-		listener( owner, self, (void*) args[0] /* handle */ );
-}
-
 static void mlt_thread_join( mlt_consumer self )
 {
 	consumer_private *priv = self->local;
-	if ( mlt_events_fire( MLT_CONSUMER_PROPERTIES(self), "consumer-thread-join", priv->ahead_thread, NULL ) < 1 )
+	if ( mlt_events_fire( MLT_CONSUMER_PROPERTIES(self), "consumer-thread-join",
+		mlt_event_data_from_object(priv->ahead_thread) ) < 1 )
 	{
 		pthread_t *handle = priv->ahead_thread;
 		pthread_join( *handle, NULL );
