@@ -2,7 +2,7 @@
  * \file mlt_factory.c
  * \brief the factory method interfaces
  *
- * Copyright (C) 2003-2019 Meltytech, LLC
+ * Copyright (C) 2003-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -44,18 +44,25 @@
    /** the default subdirectory of the install prefix for holding module (plugin) data */
 #  define PREFIX_DATA "\\share\\mlt"
 
-#elif defined(__APPLE__) && defined(RELOCATABLE)
-#  include <mach-o/dyld.h>
+#elif defined(RELOCATABLE)
 #  ifdef PREFIX_LIB
 #    undef PREFIX_LIB
 #  endif
 #  ifdef PREFIX_DATA
 #    undef PREFIX_DATA
 #  endif
-   /** the default subdirectory of the libdir for holding modules (plugins) */
-#  define PREFIX_LIB "/PlugIns/mlt"
-   /** the default subdirectory of the install prefix for holding module (plugin) data */
-#  define PREFIX_DATA "/Resources/mlt"
+#  ifdef __APPLE__
+#    include <mach-o/dyld.h>
+     /** the default subdirectory of the libdir for holding modules (plugins) */
+#    define PREFIX_LIB "/PlugIns/mlt"
+     /** the default subdirectory of the install prefix for holding module (plugin) data */
+#    define PREFIX_DATA "/Resources/mlt"
+#  else
+#    include <unistd.h>
+#    define PREFIX_LIB "/lib/mlt-7"
+     /** the default subdirectory of the install prefix for holding module (plugin) data */
+#    define PREFIX_DATA "/share/mlt-7"
+#  endif
 #endif
 
 /** holds the full path to the modules directory - initialized and retained for the entire session */
@@ -69,37 +76,7 @@ static mlt_properties event_object = NULL;
 /** for tracking the unique_id set on each constructed service */
 static int unique_id = 0;
 
-/* Event transmitters. */
-
-/** the -create-request event transmitter
- *
- * \param listener
- * \param owner
- * \param self
- * \param args
- */
-
-static void mlt_factory_create_request( mlt_listener listener, mlt_properties owner, mlt_service self, void **args )
-{
-	if ( listener != NULL )
-		listener( owner, self, ( char * )args[ 0 ], ( char * )args[ 1 ], ( mlt_service * )args[ 2 ] );
-}
-
-/** the -create-done event transmitter
- *
- * \param listener
- * \param owner
- * \param self
- * \param args
- */
-
-static void mlt_factory_create_done( mlt_listener listener, mlt_properties owner, mlt_service self, void **args )
-{
-	if ( listener != NULL )
-		listener( owner, self, ( char * )args[ 0 ], ( char * )args[ 1 ], ( mlt_service )args[ 2 ] );
-}
-
-#if defined(_WIN32) || (defined(__APPLE__) && defined(RELOCATABLE))
+#if defined(_WIN32) || defined(RELOCATABLE)
 // Replacement for buggy dirname() on some systems.
 // https://github.com/mltframework/mlt/issues/285
 static char* mlt_dirname( char *path )
@@ -167,10 +144,14 @@ mlt_repository mlt_factory_init( const char *directory )
 		#endif
 		mlt_properties_set( global_properties, "MLT_APPDIR", appdir );
 		free( appdir );
-#elif defined(__APPLE__)  && defined(RELOCATABLE)
+#elif defined(RELOCATABLE)
 		char path[1024];
 		uint32_t size = sizeof( path );
+#  ifdef __APPLE__
 		_NSGetExecutablePath( path, &size );
+#  else
+		readlink( "/proc/self/exe", path, size );
+#  endif
 		char *appdir = mlt_dirname( mlt_dirname( strdup( path ) ) );
 		mlt_properties_set( global_properties, "MLT_APPDIR", appdir );
 		free( appdir );
@@ -180,14 +161,16 @@ mlt_repository mlt_factory_init( const char *directory )
 	// Only initialise once
 	if ( mlt_directory == NULL )
 	{
-#if !defined(_WIN32) && !(defined(__APPLE__) && defined(RELOCATABLE))
+#if !defined(_WIN32) && !defined(RELOCATABLE)
 		// Allow user overrides
 		if ( directory == NULL || !strcmp( directory, "" ) )
 			directory = getenv( "MLT_REPOSITORY" );
-#endif
-
+		// If no directory is specified, default to install directory
+		if ( directory == NULL )
+			directory = PREFIX_LIB;
 		// Store the prefix for later retrieval
-#if defined(_WIN32) || (defined(__APPLE__) && defined(RELOCATABLE))
+		mlt_directory = strdup( directory );
+#else
 		if ( directory ) {
 			mlt_directory = strdup( directory );
 		} else {
@@ -205,11 +188,6 @@ mlt_repository mlt_factory_init( const char *directory )
 			strcpy( mlt_directory, exedir );
 			strcat( mlt_directory, PREFIX_LIB );
 		}
-#else
-		// If no directory is specified, default to install directory
-		if ( directory == NULL )
-			directory = PREFIX_LIB;
-		mlt_directory = strdup( directory );
 #endif
 		
 		// Initialise the pool
@@ -218,14 +196,14 @@ mlt_repository mlt_factory_init( const char *directory )
 		// Create and set up the events object
 		event_object = mlt_properties_new( );
 		mlt_events_init( event_object );
-		mlt_events_register( event_object, "producer-create-request", ( mlt_transmitter )mlt_factory_create_request );
-		mlt_events_register( event_object, "producer-create-done", ( mlt_transmitter )mlt_factory_create_done );
-		mlt_events_register( event_object, "filter-create-request", ( mlt_transmitter )mlt_factory_create_request );
-		mlt_events_register( event_object, "filter-create-done", ( mlt_transmitter )mlt_factory_create_done );
-		mlt_events_register( event_object, "transition-create-request", ( mlt_transmitter )mlt_factory_create_request );
-		mlt_events_register( event_object, "transition-create-done", ( mlt_transmitter )mlt_factory_create_done );
-		mlt_events_register( event_object, "consumer-create-request", ( mlt_transmitter )mlt_factory_create_request );
-		mlt_events_register( event_object, "consumer-create-done", ( mlt_transmitter )mlt_factory_create_done );
+		mlt_events_register( event_object, "producer-create-request" );
+		mlt_events_register( event_object, "producer-create-done" );
+		mlt_events_register( event_object, "filter-create-request" );
+		mlt_events_register( event_object, "filter-create-done" );
+		mlt_events_register( event_object, "transition-create-request" );
+		mlt_events_register( event_object, "transition-create-done" );
+		mlt_events_register( event_object, "consumer-create-request" );
+		mlt_events_register( event_object, "consumer-create-done" );
 
 		// Create the repository of services
 		repository = mlt_repository_init( mlt_directory );
@@ -354,17 +332,30 @@ mlt_producer mlt_factory_producer( mlt_profile profile, const char *service, con
 		service = mlt_environment( "MLT_PRODUCER" );
 
 	// Offer the application the chance to 'create'
-	mlt_events_fire( event_object, "producer-create-request", service, resource, &obj, NULL );
+	mlt_factory_event_data data = {
+	    .name = service,
+	    .input = resource,
+	    .service = &obj
+	};
+	mlt_events_fire( event_object, "producer-create-request", mlt_event_data_from_object(&data) );
 
 	// Try to instantiate via the specified service
 	if ( obj == NULL )
 	{
-		obj = mlt_repository_create( repository, profile, producer_type, service, resource );
-		mlt_events_fire( event_object, "producer-create-done", service, resource, obj, NULL );
+		obj = mlt_repository_create( repository, profile, mlt_service_producer_type, service, resource );
+		mlt_events_fire( event_object, "producer-create-done", mlt_event_data_from_object(&data) );
 		if ( obj != NULL )
 		{
 			mlt_properties properties = MLT_PRODUCER_PROPERTIES( obj );
-			set_common_properties( properties, profile, "producer", service );
+			if ( mlt_service_identify( MLT_PRODUCER_SERVICE( obj ) ) == mlt_service_chain_type )
+			{
+				// XML producer may return a chain
+				set_common_properties( properties, profile, "chain", service );
+			}
+			else
+			{
+				set_common_properties( properties, profile, "producer", service );
+			}
 		}
 	}
 	return obj;
@@ -383,12 +374,17 @@ mlt_filter mlt_factory_filter( mlt_profile profile, const char *service, const v
 	mlt_filter obj = NULL;
 
 	// Offer the application the chance to 'create'
-	mlt_events_fire( event_object, "filter-create-request", service, input, &obj, NULL );
+	mlt_factory_event_data data = {
+	    .name = service,
+	    .input = input,
+	    .service = &obj
+	};
+	mlt_events_fire( event_object, "filter-create-request", mlt_event_data_from_object(&data) );
 
 	if ( obj == NULL )
 	{
-   		obj = mlt_repository_create( repository, profile, filter_type, service, input );
-		mlt_events_fire( event_object, "filter-create-done", service, input, obj, NULL );
+		obj = mlt_repository_create( repository, profile, mlt_service_filter_type, service, input );
+		mlt_events_fire( event_object, "filter-create-done", mlt_event_data_from_object(&data) );
 	}
 
 	if ( obj != NULL )
@@ -399,6 +395,39 @@ mlt_filter mlt_factory_filter( mlt_profile profile, const char *service, const v
 	return obj;
 }
 
+/** Fetch a link from the repository.
+ *
+ * \param service the name of the link
+ * \param input an optional argument to the link constructor, typically a string
+ * \return a new link
+ */
+
+mlt_link mlt_factory_link( const char *service, const void *input )
+{
+	mlt_link obj = NULL;
+
+	// Offer the application the chance to 'create'
+	mlt_factory_event_data data = {
+	    .name = service,
+	    .input = input,
+	    .service = &obj
+	};
+	mlt_events_fire( event_object, "link-create-request", mlt_event_data_from_object(&data) );
+
+	if ( obj == NULL )
+	{
+		obj = mlt_repository_create( repository, NULL, mlt_service_link_type, service, input );
+		mlt_events_fire( event_object, "link-create-done", mlt_event_data_from_object(&data) );
+	}
+
+	if ( obj != NULL )
+	{
+		mlt_properties properties = MLT_LINK_PROPERTIES( obj );
+		set_common_properties( properties, NULL, "link", service );
+	}
+	return obj;
+}
+
 /** Fetch a transition from the repository.
  *
  * \param profile the \p mlt_profile to use
@@ -412,12 +441,17 @@ mlt_transition mlt_factory_transition( mlt_profile profile, const char *service,
 	mlt_transition obj = NULL;
 
 	// Offer the application the chance to 'create'
-	mlt_events_fire( event_object, "transition-create-request", service, input, &obj, NULL );
+	mlt_factory_event_data data = {
+	    .name = service,
+	    .input = input,
+	    .service = &obj
+	};
+	mlt_events_fire( event_object, "transition-create-request", mlt_event_data_from_object(&data) );
 
 	if ( obj == NULL )
 	{
-   		obj = mlt_repository_create( repository, profile, transition_type, service, input );
-		mlt_events_fire( event_object, "transition-create-done", service, input, obj, NULL );
+		obj = mlt_repository_create( repository, profile, mlt_service_transition_type, service, input );
+		mlt_events_fire( event_object, "transition-create-done", mlt_event_data_from_object(&data) );
 	}
 
 	if ( obj != NULL )
@@ -444,11 +478,16 @@ mlt_consumer mlt_factory_consumer( mlt_profile profile, const char *service, con
 		service = mlt_environment( "MLT_CONSUMER" );
 
 	// Offer the application the chance to 'create'
-	mlt_events_fire( event_object, "consumer-create-request", service, input, &obj, NULL );
+	mlt_factory_event_data data = {
+	    .name = service,
+	    .input = input,
+	    .service = &obj
+	};
+	mlt_events_fire( event_object, "consumer-create-request",mlt_event_data_from_object(&data) );
 
 	if ( obj == NULL )
 	{
-		obj = mlt_repository_create( repository, profile, consumer_type, service, input );
+		obj = mlt_repository_create( repository, profile, mlt_service_consumer_type, service, input );
 	}
 
 	if ( obj == NULL )
@@ -456,19 +495,19 @@ mlt_consumer mlt_factory_consumer( mlt_profile profile, const char *service, con
 		if ( !strcmp( service, "sdl2" ) )
 		{
 			service = "sdl";
-			obj = mlt_repository_create( repository, profile, consumer_type, service, input );
+			obj = mlt_repository_create( repository, profile, mlt_service_consumer_type, service, input );
 		}
 		else if ( !strcmp( service, "sdl_audio" ) )
 		{
 			service = "sdl2_audio";
-			obj = mlt_repository_create( repository, profile, consumer_type, service, input );
+			obj = mlt_repository_create( repository, profile, mlt_service_consumer_type, service, input );
 		}
 	}
 
 	if ( obj != NULL )
 	{
 		mlt_properties properties = MLT_CONSUMER_PROPERTIES( obj );
-		mlt_events_fire( event_object, "consumer-create-done", service, input, obj, NULL );
+		mlt_events_fire( event_object, "consumer-create-done", mlt_event_data_from_object(&data) );
 		set_common_properties( properties, profile, "consumer", service );
 	}
 	return obj;
