@@ -1,7 +1,6 @@
 /**
  * MltFilter.h - MLT Wrapper
- * Copyright (C) 2004-2015 Meltytech, LLC
- * Author: Charles Yates <charles.yates@gmail.com>
+ * Copyright (C) 2004-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,13 +14,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef MLTPP_FILTER_H
 #define MLTPP_FILTER_H
 
-#include "config.h"
+#include "MltConfig.h"
 
 #include <framework/mlt.h>
 
@@ -38,11 +37,16 @@ namespace Mlt
 		private:
 			mlt_filter instance;
 		public:
+			Filter();
 			Filter( Profile& profile, const char *id, const char *service = NULL );
+			Filter( mlt_profile profile, const char *id, const char *service = NULL );
 			Filter( Service &filter );
 			Filter( Filter &filter );
+			Filter( const Filter &filter );
 			Filter( mlt_filter filter );
+			Filter( Filter *filter );
 			virtual ~Filter( );
+			Filter& operator=( const Filter &filter );
 			virtual mlt_filter get_filter( );
 			mlt_service get_service( );
 			int connect( Service &service, int index = 0 );
