@@ -15,13 +15,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef MLTPP_FILTERED_PRODUCER_H
 #define MLTPP_FILTERED_PRODUCER_H
 
-#include "config.h"
+#include "MltConfig.h"
 
 #include "MltProducer.h"
 #include "MltFilter.h"
