@@ -62,15 +62,7 @@ static int link_get_audio( mlt_frame frame, void** audio, mlt_audio_format* form
 
 	// Validate the request
 	*channels = *channels <= 0 ? 2 : *channels;
-	int src_frequency = mlt_properties_get_int( MLT_FRAME_PROPERTIES( frame ), "audio_frequency" );
-	if ( src_frequency > 0 )
-	{
-		*frequency = src_frequency;
-	}
-	else if ( *frequency <= 0 )
-	{
-		*frequency = 48000;
-	}
+	*frequency = *frequency <= 0 ? 48000 : *frequency;
 
 	if ( source_speed < 0.1 || source_speed > 10 )
 	{
@@ -84,6 +76,12 @@ static int link_get_audio( mlt_frame frame, void** audio, mlt_audio_format* form
 		return 0;
 	}
 
+	int src_frequency = mlt_properties_get_int( MLT_FRAME_PROPERTIES( frame ), "audio_frequency" );
+	if ( src_frequency > 0 )
+	{
+		*frequency = src_frequency;
+	}
+
 	// Calculate the samples to get from the input frames
 	int link_sample_count = mlt_audio_calculate_frame_samples( link_fps, *frequency, mlt_frame_get_position( frame ) );
 	int sample_count = lrint( (double)link_sample_count * source_speed );
