@@ -1,6 +1,6 @@
 /*
  * producer_consumer.c -- produce as a consumer of an encapsulated producer
- * Copyright (C) 2008-2020 Meltytech, LLC
+ * Copyright (C) 2008-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -104,7 +104,7 @@ static int get_audio( mlt_frame frame, void **buffer, mlt_audio_format *format,
 	return result;
 }
 
-static void property_changed( mlt_properties owner, mlt_consumer self, char *name )
+static void property_changed( mlt_properties owner, mlt_consumer self, mlt_event_data event_data )
 {
 	mlt_properties properties = MLT_PRODUCER_PROPERTIES(self);
 	context cx = mlt_properties_get_data( properties, "context", NULL );
@@ -112,11 +112,12 @@ static void property_changed( mlt_properties owner, mlt_consumer self, char *nam
 	if ( !cx )
 		return;
 
-	if ( name == strstr( name, CONSUMER_PROPERTIES_PREFIX ) )
+	const char *name = mlt_event_data_to_string(event_data);
+	if ( name && name == strstr( name, CONSUMER_PROPERTIES_PREFIX ) )
 		mlt_properties_set(MLT_CONSUMER_PROPERTIES( cx->consumer ), name + strlen( CONSUMER_PROPERTIES_PREFIX ),
 			mlt_properties_get( properties, name ));
 
-	if ( name == strstr( name, PRODUCER_PROPERTIES_PREFIX ) )
+	if ( name && name == strstr( name, PRODUCER_PROPERTIES_PREFIX ) )
 		mlt_properties_set(MLT_PRODUCER_PROPERTIES( cx->producer ), name + strlen( PRODUCER_PROPERTIES_PREFIX ),
 			mlt_properties_get( properties, name ));
 }
