@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <framework/mlt_filter.h>
@@ -57,9 +57,10 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 
 		if ( mask )
 		{
-			uint8_t *alpha = mlt_frame_get_alpha_mask( frame );
 			int size = *width * *height;
+			uint8_t* alpha = mlt_pool_alloc( size );
 			memset( alpha, mask, size );
+			mlt_frame_set_alpha( frame, alpha, size, mlt_pool_release );
 		}
 	}
 
