@@ -821,7 +821,7 @@ void drawKdenliveTitle( producer_ktitle self, mlt_frame frame, mlt_image_format
 			}
 		}
 		p1.end();
-		self->format = mlt_image_rgb24a;
+		self->format = mlt_image_rgba;
 
 		convert_qimage_to_mlt_rgba(&img, self->rgba_image, width, height);
 		self->current_image = (uint8_t *) mlt_pool_alloc( image_size );
@@ -841,15 +841,15 @@ void drawKdenliveTitle( producer_ktitle self, mlt_frame frame, mlt_image_format
 	}
 
 	// Convert image to requested format
-	if ( format != mlt_image_none && format != mlt_image_glsl && format != self->format )
+	if ( format != mlt_image_none && format != mlt_image_movit && format != self->format )
 	{
 		uint8_t *buffer = NULL;
-		if ( self->format != mlt_image_rgb24a ) {
+		if ( self->format != mlt_image_rgba ) {
 			// Image buffer was previously converted, revert to original rgba buffer
 			self->current_image = (uint8_t *) mlt_pool_alloc( image_size );
 			memcpy(self->current_image, self->rgba_image, image_size);
 			mlt_properties_set_data( producer_props, "_cached_image", self->current_image, image_size, mlt_pool_release, NULL );
-			self->format = mlt_image_rgb24a;
+			self->format = mlt_image_rgba;
 		}
 
 		// First, set the image so it can be converted when we get it
