@@ -1,8 +1,6 @@
 /*
  * producer_image.c -- a QT/QImage based producer for MLT
- *
- * NB: This module is designed to be functionally equivalent to the 
- * gtk2 image loading module so it can be used as replacement.
+ * Copyright (C) 2006-2021 Meltytech, LLC
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,9 +46,10 @@ static void refresh_length( mlt_properties properties, producer_qimage self )
 	}
 }
 
-static void on_property_changed( mlt_service owner, mlt_producer producer, char *name )
+static void on_property_changed( mlt_service owner, mlt_producer producer, mlt_event_data event_data )
 {
-	if ( !strcmp( name, "ttl" ) )
+	const char *name = mlt_event_data_to_string(event_data);
+	if ( name && !strcmp( name, "ttl" ) )
 		refresh_length( MLT_PRODUCER_PROPERTIES(producer), producer->child );
 }
 
@@ -337,7 +336,7 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 		mlt_frame_set_position( *frame, mlt_producer_position( producer ) );
 
 		// Refresh the image
-		if ( self->count == 1 || mlt_properties_get_int( properties, "ttl" ) > 1 )
+		if ( self->count == 1 || mlt_properties_get_int( producer_properties, "ttl" ) > 1 )
 		{
 			self->qimage_cache = mlt_service_cache_get( MLT_PRODUCER_SERVICE( producer ), "qimage.qimage" );
 			self->qimage = mlt_cache_item_data( self->qimage_cache, NULL );
