@@ -1,6 +1,6 @@
 /*
  * consumer_sdl.c -- A Simple DirectMedia Layer consumer
- * Copyright (C) 2003-2019 Meltytech, LLC
+ * Copyright (C) 2003-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -156,7 +156,7 @@ mlt_consumer consumer_sdl_init( mlt_profile profile, mlt_service_type type, cons
 		parent->purge = consumer_purge;
 
 		// Register specific events
-		mlt_events_register( self->properties, "consumer-sdl-event", ( mlt_transmitter )consumer_sdl_event );
+		mlt_events_register( self->properties, "consumer-sdl-event" );
 
 		// Return the consumer produced
 		return parent;
@@ -169,12 +169,6 @@ mlt_consumer consumer_sdl_init( mlt_profile profile, mlt_service_type type, cons
 	return NULL;
 }
 
-static void consumer_sdl_event( mlt_listener listener, mlt_properties owner, mlt_service self, void **args )
-{
-	if ( listener != NULL )
-		listener( owner, self, ( SDL_Event * )args[ 0 ] );
-}
-
 int consumer_start( mlt_consumer parent )
 {
 	consumer_sdl self = parent->child;
@@ -551,7 +545,7 @@ static int consumer_play_video( consumer_sdl self, mlt_frame frame )
 
 			while ( SDL_PollEvent( &event ) )
 			{
-				mlt_events_fire( self->properties, "consumer-sdl-event", &event, NULL );
+				mlt_events_fire( self->properties, "consumer-sdl-event", mlt_event_data_from_object(&event) );
 
 				switch( event.type )
 				{
@@ -682,9 +676,7 @@ static int consumer_play_video( consumer_sdl self, mlt_frame frame )
 			self->buffer = self->sdl_overlay->pixels[ 0 ];
 			if ( SDL_LockYUVOverlay( self->sdl_overlay ) >= 0 )
 			{
-				// We use height-1 because mlt_image_format_size() uses height + 1.
-				// XXX Remove -1 when mlt_image_format_size() is changed.
-				int size = mlt_image_format_size( vfmt, width, height - 1, NULL );
+				int size = mlt_image_format_size( vfmt, width, height, NULL );
 				if ( image != NULL )
 					memcpy( self->buffer, image, size );
 				SDL_UnlockYUVOverlay( self->sdl_overlay );
@@ -694,14 +686,14 @@ static int consumer_play_video( consumer_sdl self, mlt_frame frame )
 
 		sdl_unlock_display();
 		mlt_cocoa_autorelease_close( pool );
-		mlt_events_fire( properties, "consumer-frame-show", frame, NULL );
+		mlt_events_fire( properties, "consumer-frame-show", mlt_event_data_from_frame(frame) );
 	}
 	else if ( self->running )
 	{
-		vfmt = preview_format == mlt_image_none ? mlt_image_rgb24a : preview_format;
+		vfmt = preview_format == mlt_image_none ? mlt_image_rgba : preview_format;
 		if ( !video_off )
 			mlt_frame_get_image( frame, &image, &vfmt, &width, &height, 0 );
-		mlt_events_fire( properties, "consumer-frame-show", frame, NULL );
+		mlt_events_fire( properties, "consumer-frame-show", mlt_event_data_from_frame(frame) );
 	}
 
 	return 0;
