@@ -1,6 +1,6 @@
 /*
  * consumer_sdl_preview.c -- A Simple DirectMedia Layer consumer
- * Copyright (C) 2004-2014 Meltytech, LLC
+ * Copyright (C) 2004-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -61,9 +61,9 @@ static int consumer_is_stopped( mlt_consumer parent );
 static void consumer_purge( mlt_consumer parent );
 static void consumer_close( mlt_consumer parent );
 static void *consumer_thread( void * );
-static void consumer_frame_show_cb( mlt_consumer sdl, mlt_consumer self, mlt_frame frame );
-static void consumer_sdl_event_cb( mlt_consumer sdl, mlt_consumer self, SDL_Event *event );
-static void consumer_refresh_cb( mlt_consumer sdl, mlt_consumer self, char *name );
+static void consumer_frame_show_cb( mlt_consumer sdl, mlt_consumer self, mlt_event_data );
+static void consumer_sdl_event_cb( mlt_consumer sdl, mlt_consumer self, mlt_event_data );
+static void consumer_refresh_cb(mlt_consumer sdl, mlt_consumer self, mlt_event_data );
 
 mlt_consumer consumer_sdl_preview_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg )
 {
@@ -108,29 +108,33 @@ mlt_consumer consumer_sdl_preview_init( mlt_profile profile, mlt_service_type ty
 		pthread_cond_init( &self->refresh_cond, NULL );
 		pthread_mutex_init( &self->refresh_mutex, NULL );
 		mlt_events_listen( MLT_CONSUMER_PROPERTIES( parent ), self, "property-changed", ( mlt_listener )consumer_refresh_cb );
-		mlt_events_register( properties, "consumer-sdl-paused", NULL );
+		mlt_events_register( properties, "consumer-sdl-paused" );
 		return parent;
 	}
 	free( self );
 	return NULL;
 }
 
-void consumer_frame_show_cb( mlt_consumer sdl, mlt_consumer parent, mlt_frame frame )
+void consumer_frame_show_cb(mlt_consumer sdl, mlt_consumer parent, mlt_event_data event_data )
 {
+	mlt_frame frame = mlt_event_data_to_frame(event_data);
 	consumer_sdl self = parent->child;
-	self->last_speed = mlt_properties_get_double( MLT_FRAME_PROPERTIES( frame ), "_speed" );
-	self->last_position = mlt_frame_get_position( frame );
-	mlt_events_fire( MLT_CONSUMER_PROPERTIES( parent ), "consumer-frame-show", frame, NULL );
+	if (frame && self) {
+		self->last_speed = mlt_properties_get_double( MLT_FRAME_PROPERTIES( frame ), "_speed" );
+		self->last_position = mlt_frame_get_position( frame );
+		mlt_events_fire( MLT_CONSUMER_PROPERTIES( parent ), "consumer-frame-show", event_data );
+	}
 }
 
-static void consumer_sdl_event_cb( mlt_consumer sdl, mlt_consumer parent, SDL_Event *event )
+static void consumer_sdl_event_cb( mlt_consumer sdl, mlt_consumer parent, mlt_event_data event_data )
 {
-	mlt_events_fire( MLT_CONSUMER_PROPERTIES( parent ), "consumer-sdl-event", event, NULL );
+	mlt_events_fire( MLT_CONSUMER_PROPERTIES( parent ), "consumer-sdl-event", event_data );
 }
 
-static void consumer_refresh_cb( mlt_consumer sdl, mlt_consumer parent, char *name )
+static void consumer_refresh_cb( mlt_consumer sdl, mlt_consumer parent, mlt_event_data event_data )
 {
-	if ( !strcmp( name, "refresh" ) )
+	const char *name = mlt_event_data_to_string(event_data);
+	if ( name && !strcmp( name, "refresh" ) )
 	{
 		consumer_sdl self = parent->child;
 		pthread_mutex_lock( &self->refresh_mutex );
@@ -421,7 +425,7 @@ static void *consumer_thread( void *arg )
 				}
 				if ( pause && speed == 0.0 )
 				{
-					mlt_events_fire( properties, "consumer-sdl-paused", NULL );
+					mlt_events_fire( properties, "consumer-sdl-paused", mlt_event_data_none() );
 				}
 			}
 			// Allow a little grace time before switching consumers on speed changes
@@ -462,7 +466,7 @@ static void *consumer_thread( void *arg )
 				pthread_mutex_lock( &self->refresh_mutex );
 				if ( self->running && speed == 0 && self->refresh_count <= 0 )
 				{
-					mlt_events_fire( properties, "consumer-sdl-paused", NULL );
+					mlt_events_fire( properties, "consumer-sdl-paused", mlt_event_data_none() );
 					pthread_cond_wait( &self->refresh_cond, &self->refresh_mutex );
 				}
 				self->refresh_count --;
