@@ -1,6 +1,6 @@
 /*
  * consumer_sdl.c -- A Simple DirectMedia Layer consumer
- * Copyright (C) 2017-2019 Meltytech, LLC
+ * Copyright (C) 2017-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -84,7 +84,6 @@ static int consumer_is_stopped( mlt_consumer parent );
 static void consumer_purge( mlt_consumer parent );
 static void consumer_close( mlt_consumer parent );
 static void *consumer_thread( void * );
-static void consumer_sdl_event( mlt_listener listener, mlt_properties owner, mlt_service self, void **args );
 static int setup_sdl_video( consumer_sdl self );
 
 /** This is what will be called by the factory - anything can be passed in
@@ -156,7 +155,7 @@ mlt_consumer consumer_sdl2_init( mlt_profile profile, mlt_service_type type, con
 		parent->purge = consumer_purge;
 
 		// Register specific events
-		mlt_events_register( self->properties, "consumer-sdl-event", ( mlt_transmitter )consumer_sdl_event );
+		mlt_events_register( self->properties, "consumer-sdl-event" );
 
 		// Return the consumer produced
 		return parent;
@@ -169,12 +168,6 @@ mlt_consumer consumer_sdl2_init( mlt_profile profile, mlt_service_type type, con
 	return NULL;
 }
 
-static void consumer_sdl_event( mlt_listener listener, mlt_properties owner, mlt_service self, void **args )
-{
-	if ( listener != NULL )
-		listener( owner, self, ( SDL_Event * )args[ 0 ] );
-}
-
 int consumer_start( mlt_consumer parent )
 {
 	consumer_sdl self = parent->child;
@@ -525,10 +518,10 @@ static int setup_sdl_video( consumer_sdl self )
 	int image_format = mlt_properties_get_int( self->properties, "mlt_image_format" );
 
 	if ( image_format ) switch ( image_format ) {
-	case mlt_image_rgb24:
+	case mlt_image_rgb:
 		texture_format = SDL_PIXELFORMAT_RGB24;
 		break;
-	case mlt_image_rgb24a:
+	case mlt_image_rgba:
 		texture_format = SDL_PIXELFORMAT_ABGR8888;
 		break;
 	case mlt_image_yuv420p:
@@ -668,9 +661,7 @@ static int consumer_play_video( consumer_sdl self, mlt_frame frame )
 			unsigned char* planes[4];
 			int strides[4];
 
-			// We use height-1 because mlt_image_format_size() uses height + 1.
-			// XXX Remove -1 when mlt_image_format_size() is changed.
-			mlt_image_format_planes( vfmt, width, height - 1, image, planes, strides );
+			mlt_image_format_planes( vfmt, width, height, image, planes, strides );
 			if ( strides[1] ) {
 				SDL_UpdateYUVTexture( self->sdl_texture, NULL,
 					planes[0], strides[0],
@@ -684,16 +675,16 @@ static int consumer_play_video( consumer_sdl self, mlt_frame frame )
 			SDL_RenderPresent( self->sdl_renderer );
 		}
 
-		mlt_events_fire( properties, "consumer-frame-show", frame, NULL );
+		mlt_events_fire( properties, "consumer-frame-show", mlt_event_data_from_frame(frame) );
 	}
 	else if ( self->running )
 	{
 		if ( !video_off ) {
 			mlt_image_format preview_format = mlt_properties_get_int( properties, "preview_format" );
-			vfmt = preview_format == mlt_image_none ? mlt_image_rgb24a : preview_format;
+			vfmt = preview_format == mlt_image_none ? mlt_image_rgba : preview_format;
 			mlt_frame_get_image( frame, &image, &vfmt, &width, &height, 0 );
 		}
-		mlt_events_fire( properties, "consumer-frame-show", frame, NULL );
+		mlt_events_fire( properties, "consumer-frame-show", mlt_event_data_from_frame(frame) );
 	}
 
 	return 0;
